#include "actuator.h"

#include "controller.h"

#include <avr/io.h>
#include <stdint.h>
#include <stdlib.h>

//The maximum voltage the actuator can handle
#define ACTUATOR_MAXVOLTAGE 1.5f

void ACTUATOR_init() {

	//Timer 1 (Pan)
	//Set PWM pins as output pins
	DDRB |= (1 << DDB6) | (1 << DDB5);
	
	//Waveform generation mode 14, fast PWM
	TCCR1A |= (1 << COM1A1) | (1 << COM1B1) | (1 << WGM11);
	TCCR1B |= (1 << WGM13) | (1 << WGM12);

	//Prescaler TODO: sjekk hastighet
	TCCR1B |= (1 << CS10);

	//TOP, i.e. the number of clock cycles (times prescaler) for each period
	//TODO sjekk dette
	ICR1 = PWM_PERIOD;

	//Timer 3 (Tilt)
	//Set PWM pins as output pins
	DDRE |= (1 << DDE4) | (1 << DDE3);

	//Waveform generation mode 14, fast PWM
	TCCR3A |= (1 << COM3A1) | (1 << COM3B1) | (1 << WGM31);
	TCCR3B |= (1 << WGM33) | (1 << WGM32);

	//Prescaler TODO: sjekk hastighet
	TCCR3B |= (1 << CS30);

	//TOP, i.e. the number of clock cycles (times prescaler) for each period
	//TODO sjekk dette
	ICR3 = PWM_PERIOD;
}

//Value is a voltage between -VCC and +VCC
//Returns the voltage actually set.
float ACTUATOR_setValue(uint8_t axis, float value) {

	if(value >= ACTUATOR_MAXVOLTAGE) {
		value = ACTUATOR_MAXVOLTAGE;
	}
	if(value <= -ACTUATOR_MAXVOLTAGE) {
		value = -ACTUATOR_MAXVOLTAGE;
	}

	int16_t actValue = (int16_t)(value*PWM_PERIOD/VCC);

	switch(axis) {

		case ACTUATOR_T:

			if(value < 0) {
				//Set reverse PWM output to zero.
				//Turn off PWM, then set output to zero
				TCCR1A &= ~(1 << COM1A1);
				PORTB &= ~(1 << PB5);

				//Turn on PWM in forward direction
				TCCR1A |= (1 << COM1B1);

				//Set output compare to value
				OCR1B = abs(actValue);

			} else if(value > 0) {
				//Set forward PWM output to zero.
				//Turn off PWM, then set output to zero
				TCCR1A &= ~(1 << COM1B1);
				PORTB &= ~(1 << PB6);

				//Turn on PWM in reverse direction
				TCCR1A |= (1 << COM1A1);

				//Set output compare to value
				OCR1A = abs(actValue);

			} else if(value == 0) {
				//Set PWM output to zero in both directions
				//Turn off PWM, then set output to zero
				TCCR1A &= ~(1 << COM1B1);
				PORTB &= ~(1 << PB6);
				TCCR1A &= ~(1 << COM1A1);
				PORTB &= ~(1 << PB5);

			}
			break;

		case ACTUATOR_P:
		if(value < 0) {
				//Set reverse PWM output to zero.
				//Turn off PWM, then set output to zero
				TCCR3A &= ~(1 << COM3A1);
				PORTE &= ~(1 << PB3);

				//Turn on PWM in forward direction
				TCCR3A |= (1 << COM3B1);

				//Set output compare to value
				OCR3B = abs(actValue);

			} else if(value > 0) {
				//Set forward PWM output to zero.
				//Turn off PWM, then set output to zero
				TCCR3A &= ~(1 << COM3B1);
				PORTE &= ~(1 << PE4);

				//Turn on PWM in reverse direction
				TCCR3A |= (1 << COM3A1);

				//Set output compare to value
				OCR3A = abs(actValue);

			} else if(value == 0) {
				//Set PWM output to zero in both directions
				//Turn off PWM, then set output to zero
				TCCR3A &= ~(1 << COM3B1);
				PORTE &= ~(1 << PE4);
				TCCR3A &= ~(1 << COM3A1);
				PORTE &= ~(1 << PE3);

			}

			break;

	}
	return value;
}
