#include "gyro.h"

#include "adc.h"

#include <stdint.h>
#include <stdio.h>
#include <util/delay.h>

#define PI 3.14159f

static float center_p = 388;
static float center_t;

/*
Calibrate gyro, read the output from the gyro many times while
the gyro is not rotating, take the average of these outputs
as the zero-level output.
*/
void GYRO_calibrate(uint8_t axis) {

	float center = 0;
	int i;
	int n = 100;

	switch(axis) {
		case GYRO_P:
			for(i = 0; i < n; i++) {
				center += ADC_getValue(ADC1);
				_delay_ms(10);
			}
			center /= n;
			center_p = center;
			break;
		case GYRO_T:
			for(i = 0; i < n; i++) {
				center += ADC_getValue(ADC0);
				_delay_ms(10);
			}
			center /= n;
			center_p = center;
			break;
	}
}

float GYRO_getValue(uint8_t axis) {

	static float pan = 0;
	static float tilt = 0;

	uint16_t adcvalue = 0;

	switch(axis) {
		case GYRO_P:
			adcvalue = ADC_getValue(ADC1);

			//Integrate angular velocity to get angle
			pan += ((float)adcvalue - center_p) * PI/180.0f * GYRO_TS;

			return pan;
		case GYRO_T:
			adcvalue = ADC_getValue(ADC0);

			//Integrate angular velocity to get angle
			tilt += ((float)adcvalue - center_t) * PI/180 * GYRO_TS;

			return tilt;
	}
	return 0.0;
}
