#include "hall.h"

#include "adc.h"

#include <stdint.h>
#include <stdio.h>

//Lookup table increment
#define LUT_P_INC 20
#define LUT_P_MAX 760
#define LUT_P_MIN 340

#define LUT_T_INC 20
#define LUT_T_MAX 760
#define LUT_T_MIN 340

//Lookup table of ADC values between LUT_MIN and LUT_MAX, with increments of LUT_INC
float lookuptable_p[22] = {
-0.4451,
-0.4189,
-0.3840,
-0.3491,
-0.3054,
-0.2705,
-0.2356,
-0.2007,
-0.1658,
-0.1222,
-0.0873,
-0.0436,
0.0000,
0.0436,
0.0873,
0.1309,
0.1745,
0.2182,
0.2618,
0.3054,
0.3578,
0.4189
};


float HALL_getValue(uint8_t axis) {

	uint16_t adcvalue = 0;
	float position = 0;
	uint16_t index = 0;

	switch(axis) {
		case HALL_P:
			adcvalue = ADC_getValue(ADC3);


			//If the measurement is outside of the range of the lookup table,
			//use the extreme values of the lookup table
			if(adcvalue > LUT_P_MAX) adcvalue = LUT_P_MAX;
			if(adcvalue < LUT_P_MIN) adcvalue = LUT_P_MIN;

			//Find the two nearest values in the lookup table, then
			//use linear interpolation between them
			index = (adcvalue - LUT_P_MIN) / LUT_P_INC;
			position = (lookuptable_p[index] + 
			(lookuptable_p[index + 1] - lookuptable_p[index]) * 
			(adcvalue % LUT_P_INC) / LUT_P_INC) * (-1);


			break;
		case HALL_T:
			//Uses same lookup table as pan axis. Needs to be fixed when 
			//extending the system to two axes.
			adcvalue = ADC_getValue(ADC2);
			
			//If the measurement is outside of the range of the lookup table,
			//use the extreme values of the lookup table
			if(adcvalue > LUT_T_MAX) adcvalue = LUT_T_MAX;
			if(adcvalue < LUT_T_MIN) adcvalue = LUT_T_MIN;

			//Find the two nearest values in the lookup table, then
			//use linear interpolation between them
			index = (adcvalue - LUT_T_MIN) / LUT_T_INC;
			position = (lookuptable_p[index] + 
			(lookuptable_p[index + 1] - lookuptable_p[index]) * 
			(adcvalue % LUT_T_INC) / LUT_T_INC) * (-1);

			break;
	}

	return position;

}
