#include <stdio.h>

#include <stdint.h>
#include <avr/io.h>
#include "uart.h"

volatile unsigned char last_received;
volatile char new_received;
	
void UART_init(){
	unsigned int brr;
	
	brr = 8; //From table 20-11 datasheet, 115.2kbps, 8Mhz clock

	/* Set baud rate */
	UBRR0H = (unsigned char)(brr>>8);
	UBRR0L = (unsigned char)brr;
	
	UCSR0A |= (1<<U2X0);
	/* Enable receiver and transmitter */
	UCSR0B = (1<<RXEN0)|(1<<TXEN0);
	
	/* Set frame format: 8data, 1stop bit */
	UCSR0C = (3<<UCSZ00);

	fdevopen (&UART_transmit, &UART_receive);
}

int UART_transmit( unsigned char data){
	/* Wait for empty transmit buffer */
	while ( !( UCSR0A & (1<<UDRE0)) );
	
	/* Put data into buffer, sends the data */
	UDR0 = data;
	return 0;
}

unsigned char UART_receive(){
	/* Wait for data to be received for a little time */
	unsigned char count = 0;
	while ( !(UCSR0A & (1<<RXC0)) ){
		count++;
		if(count>240){
			return -1;
		}	
	}
	
	/* Get and return received data from buffer */
	return UDR0;
}


